# -*- coding: utf-8 -*-

import sys
import logging
import optparse
import multiprocessing

import salt_confd.version

from salt.ext import six
import salt.version
import salt.utils.args
import salt.utils.parsers
import salt.config as config


def salt_information():
    '''
    Return version of Salt and salt-sproxy.
    '''
    yield 'Salt', salt.version.__version__
    yield 'Salt Confd', salt_confd.version.__version__


def dependency_information(include_salt_cloud=False):
    '''
    Report versions of library dependencies.

    This function has been ported from
    https://github.com/saltstack/salt/blob/develop/salt/version.py
    and extended here to collect the version information for several more
    libraries that may be necessary for various Proxy (or Execution) Modules.
    '''
    libs = [
        ('Python', None, sys.version.rsplit('\n')[0].strip()),
        ('timelib', 'timelib', 'version'),
        ('dateutil', 'dateutil', '__version__'),
        ('gitpython', 'git', '__version__'),
    ]
    if include_salt_cloud:
        libs.append(('Apache Libcloud', 'libcloud', '__version__'))

    for name, imp, attr in libs:
        if imp is None:
            yield name, attr
            continue
        try:
            imp = __import__(imp)
            version = getattr(imp, attr)
            if callable(version):
                version = version()
            if isinstance(version, (tuple, list)):
                version = '.'.join(map(str, version))
            yield name, version
        except Exception:
            yield name, None

salt.version.salt_information = salt_information
salt.version.dependency_information = dependency_information


class SaltConfdOptionParser(six.with_metaclass(salt.utils.parsers.OptionParserMeta,
                                              salt.utils.parsers.OptionParser,
                                              salt.utils.parsers.ConfigDirMixIn,
                                              salt.utils.parsers.ExecutorsMixIn,
                                              salt.utils.parsers.MergeConfigMixIn,
                                              salt.utils.parsers.LogLevelMixIn,
                                              salt.utils.parsers.OutputOptionsMixIn,
                                              salt.utils.parsers.HardCrashMixin,
                                              salt.utils.parsers.SaltfileMixIn,
                                              salt.utils.parsers.ArgsStdinMixIn,
                                              salt.utils.parsers.ProfilingPMixIn,
                                              salt.utils.parsers.NoParseMixin,
                                              salt.utils.parsers.CacheDirMixIn)):

    VERSION = salt_confd.version.__version__

    epilog = (
        'You can find additional help about %prog at '
        'https://salt-confd.readthedocs.io/en/latest/'
    )

    description = (
        'salt-confd is used to manage configuration files for local apps'
    )

    usage = '%prog [options]'

    # ConfigDirMixIn config filename attribute
    _config_filename_ = 'confd.yml'

    # LogLevelMixIn attributes
    _default_logging_level_ = config.DEFAULT_MINION_OPTS['log_level']
    _default_logging_logfile_ = config.DEFAULT_MINION_OPTS['log_file']

    def _mixin_setup(self):
        self.add_option(
            '--return',
            default='',
            metavar='RETURNER',
            help=('Set salt-call to pass the return data to one or many '
                  'returner interfaces.')
        )
        self.add_option(
            '--file-root',
            default=None,
            help='Set this directory as the base file root.'
        )
        self.add_option(
            '--pillar-root',
            default=None,
            help='Set this directory as the base pillar root.'
        )
        self.add_option(
            '--confdir',
            default='/etc/salt/confd',
            help='The Salt confd conf directory'
        )
        self.add_option(
            '--retcode-passthrough',
            default=False,
            action='store_true',
            help=('Exit with the salt call retcode and not the salt binary '
                  'retcode.')
        )
        self.add_option(
            '--metadata',
            default=False,
            dest='print_metadata',
            action='store_true',
            help=('Print out the execution metadata as well as the return. '
                  'This will print out the outputter data, the return code, '
                  'etc.')
        )
        self.add_option(
            '--set-metadata',
            dest='metadata',
            default=None,
            metavar='METADATA',
            help=('Pass metadata into Salt, used to search jobs.')
        )
        self.add_option(
            '--skip-grains',
            default=False,
            action='store_true',
            help=('Do not load grains.')
        )
        self.add_option(
            '--refresh-grains-cache',
            default=False,
            action='store_true',
            help=('Force a refresh of the grains cache.')
        )
        self.add_option(
            '--test', '--dry-run', '--noop',
            dest='test',
            action='store_true',
            default=False,
            help=('Dry run, show pending changes.')
        )
        self.add_option(
            '--onetime',
            dest='onetime',
            action='store_true',
            default=False,
            help=('Run once and exit.')
        )

    def setup_config(self):
        return config.minion_config(self.get_config_file_path(),
                                    cache_minion_id=True)



# -*- coding: utf-8 -*-

import sys

from salt.scripts import _install_signal_handlers


def salt_confd():
    '''
    Execute a salt convenience routine.
    '''
    import salt_confd.cli

    if '' in sys.path:
        sys.path.remove('')
    client = salt_confd.cli.SaltConfd()
    _install_signal_handlers(client)
    client.run()


if __name__ == '__main__':
    salt_confd()



# -*- coding: utf-8 -*-
# Copyright 2019 Mircea Ulinic. All rights reserved.
#
# The contents of this file are licensed under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
'''
The CLI entry point module.
'''
from __future__ import absolute_import, print_function, unicode_literals

# stdlib
import re
import os
import sys
import copy
import logging

# local modules
from salt_confd.parsers import SaltConfdOptionParser

# Salt modules
import salt.loader
import salt.cli.caller
import salt.utils.jinja
import salt.utils.stringio
import salt.utils.templates
from salt.utils.verify import verify_log

# 3rd party libs
import jinja2


log = logging.getLogger(__name__)


def getv(self, value, sort_keys=True, indent=None):
    return jinja2.Markup(value)


def parse(self, parser):
    log.error(self.environment.filters)
    self.environment.filters.update({
        'getv': self.getv
    })
    if parser.stream.current.value == 'import_yaml':
        return self.parse_yaml(parser)
    elif parser.stream.current.value == 'import_json':
        return self.parse_json(parser)
    elif parser.stream.current.value == 'import_text':
        return self.parse_text(parser)
    elif parser.stream.current.value in self._load_parsers:
        return self.parse_load(parser)
    parser.fail('Unknown format ' + parser.stream.current.value,
                parser.stream.current.lineno)

salt.utils.jinja.SerializerExtension._load_parsers.update(['getv'])
salt.utils.jinja.SerializerExtension.tags.update(['getv'])
setattr(salt.utils.jinja.SerializerExtension, 'getv', getv)
setattr(salt.utils.jinja.SerializerExtension, 'parse', parse)


class SaltConfd(SaltConfdOptionParser):
    '''
    Render the template(s) to generate the configuration files.
    '''

    def run(self):
        '''
        Execute a lightweight salt-call with bespoken options.
        '''
        self.parse_args()

        if self.options.file_root:
            # check if the argument is pointing to a file on disk
            file_root = os.path.abspath(self.options.file_root)
            self.config['file_roots'] = {'base': _expand_glob_path([file_root])}

        if self.options.pillar_root:
            # check if the argument is pointing to a file on disk
            pillar_root = os.path.abspath(self.options.pillar_root)
            self.config['pillar_roots'] = {'base': _expand_glob_path([pillar_root])}

        # Always local client
        self.config['file_client'] = 'local'
        self.config['transport'] = 'tcp'

        if not self.config.get('confdir'):
            self.config['confdir'] = os.path.join(self.config['config_dir'], 'confd')

        confd_dir = os.path.join(self.config['confdir'], 'conf.d')
        templates_dir = os.path.join(self.config['confdir'], 'templates')

        self.config['file_roots']['base'].append(os.path.dirname(__file__))
        self.config['file_roots']['base'].append(templates_dir)

        # Setup file logging!
        self.setup_logfile_logger()
        verify_log(self.config)

        self.config['fun'] = 'state.apply'
        self.config['arg'] = ['confd']
        self.config['confd'] = {}

        minion_mods = salt.loader.minion_mods(self.config)
        renderers = salt.loader.render(self.config, minion_mods)

        kwargs = {}
        path_or_string = ':string:'
        rgx = re.compile(r'^(.*):\/\/(.*)$')

        for file_ in os.listdir(confd_dir):
            confd_file = os.path.join(confd_dir, file_)
            if not os.path.isfile(confd_file):
                continue
            with salt.utils.files.fopen(confd_file, 'r') as fp_:
                string = fp_.read()
                kwargs['input_data'] = string
                ret = salt.template.compile_template(
                    path_or_string,
                    renderers,
                    self.config.get('default_renderer', 'jinja|yaml'),
                    self.config.get('renderer_blacklist'),
                    self.config.get('renderer_whitelist'),
                    **kwargs
                )
                res = ret.read() if salt.utils.stringio.is_readable(ret) else ret
                if not rgx.match(res['src']):
                    res['src'] = 'salt://' + res['src']
                if not res.get('template'):
                    res['template'] = 'jinja'
                item = os.path.splitext(file_)[0]
                self.config['confd'][item] = res

        caller = salt.cli.caller.Caller.factory(self.config)
        caller.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
The setup script for salt-confd
'''
import codecs
from setuptools import setup, find_packages

__author__ = 'Mircea Ulinic <ping@mirceaulinic.net>'

with codecs.open('pypi.rst', 'r', encoding='utf8') as file:
    long_description = file.read()

with open("requirements.txt", "r") as fs:
    reqs = [r for r in fs.read().splitlines() if (len(r) > 0 and not r.startswith("#"))]

setup(
    name='salt-confd',
    version='2019.9.0a1',
    namespace_packages=['salt_confd'],
    packages=find_packages(),
    author='Mircea Ulinic',
    author_email='ping@mirceaulinic.net',
    description='Lightweight Salt package for confd-style management of local application configuration files',
    long_description=long_description,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'License :: OSI Approved :: Apache Software License',
        'Topic :: Utilities',
        'Topic :: System :: Clustering',
        'Topic :: System :: Operating System',
        'Topic :: System :: Distributed Computing',
        'Topic :: System :: Systems Administration',
        'Programming Language :: Python',
        'Programming Language :: Cython',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: MacOS',
        'Intended Audience :: Developers',
        'Intended Audience :: Telecommunications Industry',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
    ],
    url='https://github.com/mirceaulinic/salt-confd',
    license="Apache License 2.0",
    keywords=('confd', 'docker', 'kubernetes', 'configuration', 'application'),
    include_package_data=True,
    install_requires=reqs,
    entry_points={'console_scripts': ['salt-confd=salt_confd.scripts:salt_confd']},
)
